#include "gamewidget.h"
#include "music_player.h"


GameWidget::GameWidget()
	:Widget(),
	playing(false),
	paused(false)
{
	timePanel = 0;
	NewGame();
}



void GameWidget::Draw(Bitmap &canvas) {
	Font fnt = GetFont(0);

	// sky gradient
	Gradient(canvas, 0, 0, w()-1, h(), Color(96,96,96), Color(128,128,128));
	//canvas.Clear(Color::white);

	/*
	rect(canvas, 8, 8, 8+464, 8+464, 0);
	rect(canvas, 12, 12, 12+456, 12+456, 0);
	rect(canvas, 44, 44, 44+392, 44+392, 0);
	rect(canvas, 48, 48, 48+384, 48+384, 0);
	*/

	for (int j=0; j<6; j++) {
		for (int i=0; i<6; i++) {
			blit(floor_tile, canvas, 0, 0, 48+i*64, 48+j*64, 64, 64);
		}
	}

	masked_blit(back_tile, canvas, 0, 0, 8, 8, 464, 464);

	for (int i=0; i<4; i++) {
		tank[i].Draw(canvas);
	}

	for (int i=0; i<rockets.size(); i++) {
		rockets[i].Draw(canvas);
	}

	for (int i=0; i<4; i++) {
		drone[i].Draw(canvas);
	}

	// player
	if (playing) {
		player.Draw(canvas);
	}

	for (std::vector<EXPLOSION *>::iterator e = exp.begin(); e != exp.end(); ++e) {
		draw_explosion(*e, canvas);
	}

	// overlays
	if (paused) {
		textprintf_centre_ex(canvas, focusfont, w()/2+4, 40+(h() - text_height(focusfont))/2+4, 0, -1, "Paused - (Q)uit, (C)ontinue");
		textprintf_centre_ex(canvas, focusfont, w()/2, 40+(h() - text_height(focusfont))/2, -1, -1, "Paused - (Q)uit, (C)ontinue");
	}
	else if (!game_in_progress && player.dead) {
		textprintf_centre_ex(canvas, focusfont, w()/2+4, -10+(h() - text_height(focusfont))/2+4, 0, -1, "Game over!");
		textprintf_centre_ex(canvas, focusfont, w()/2, -10+(h() - text_height(focusfont))/2, -1, -1, "Game over!");
		int time = timePanel->time;
		int min = time/(60*100);
		int sec = time/100 - min*60;
		int msec = time - sec*100 - min*60*100;
		textprintf_centre_ex(canvas, focusfont, w()/2+4, 40+(h() - text_height(focusfont))/2+4, 0, -1, "Your time: %01d:%02d.%02d", min, sec, msec);
		textprintf_centre_ex(canvas, focusfont, w()/2, 40+(h() - text_height(focusfont))/2, -1, -1, "Your time: %01d:%02d.%02d", min, sec, msec);
		if (!have_highscore) {
			textprintf_centre_ex(canvas, focusfont, w()/2+4, 90+(h() - text_height(focusfont))/2+4, 0, -1, "Press ENTER to restart");
			textprintf_centre_ex(canvas, focusfont, w()/2, 90+(h() - text_height(focusfont))/2, -1, -1, "Press ENTER to restart");
		}
		else {
			textprintf_centre_ex(canvas, focusfont, w()/2+4-60, 90+(h() - text_height(focusfont))/2+4, 0, -1, "Enter your name:");
			textprintf_centre_ex(canvas, focusfont, w()/2-60, 90+(h() - text_height(focusfont))/2, -1, -1, "Enter your name:");
		}
	}
}


void GameWidget::KillRocket(Rocket *r) {
	EXPLOSION *e = create_explosion(rocket_sprite, (int)r->pos.x, (int)r->pos.y);
	exp.push_back(e);
	play_sound(explode_sample, 128, 128, -100, 0);
}


void GameWidget::KillPlayer() {
	EXPLOSION *e = create_explosion(player_sprite[1], (int)player.pos.x, (int)player.pos.y);
	exp.push_back(e);
	play_sound(explode_sample, 255, 128, -100, 0);
	player.dead = true;
	game_in_progress = false;

DEBUG
	if (hsc->CanAdd(timePanel->time)) {
DEBUG
		have_highscore = true;
		Window dlg;
		dlg.ClearFlag(D_MOVABLE);
		EditBox edit;
		dlg.Shape(w()/2+136, 90+(h() - text_height(focusfont))/2, 176, 32);
		dlg.GetClientArea()->Resize(dlg.w(), dlg.h());
		edit.Setup(0, 0, dlg.w(), dlg.h(), 0, 0, "", 16);
		edit.MakeExit();
		for (int i=0; i<4; i++)
			edit.SetFontColor(Color::green, Color::black, i);
		dlg.Add(edit);
		dlg.Popup(GetParent()->Root(), dlg.x(), dlg.y(), &edit);
DEBUG
		HSC::Entry e;
DEBUG
		e.name = edit.GetText();
DEBUG
		e.score = timePanel->time;
		hsc->AddEntry(e);
DEBUG
		hsc->Save("dodger.hsc");
DEBUG
		GetParent()->HandleEvent(*this, 2001);
DEBUG
		have_highscore = false;
	}
DEBUG
}


void GameWidget::MsgTick() {
	Widget::MsgTick();

	if (!paused && playing) {
		player.Update();

		if (game_in_progress && !player.dead) {
			timePanel->time += 1;
		}

		if (game_in_progress) {
			for (int i=0; i<4; i++) {
				tank[i].Update();
			}

			for (int i=0; i<4; i++) {
				drone[i].Update();
			}
		}

		for (std::vector<Rocket>::iterator r = rockets.begin(); r != rockets.end(); ) {
			r->Update();

			// see if a rocket hit a wall
			if (r->pos.x == r->min.x || r->pos.x == r->max.x || r->pos.y == r->min.y || r->pos.y == r->max.y) {
				KillRocket(&*r);
				r = rockets.erase(r);
			}
			else {
				bool erased = false;

				// check collision with drones
				for (int i=0; i<4; i++) {
					if (r->CollidesWith(&drone[i])) {
						KillRocket(&*r);
						r = rockets.erase(r);
						erased = true;
						break;
					}
				}

				// check collision with player
				if (!erased && !player.dead) {
					if (r->CollidesWith(&player)) {
						KillRocket(&*r);
						r = rockets.erase(r);
						erased = true;
						KillPlayer();
						return;
					}
				}

				if (!erased) {
					++r;
				}
			}
		}

		if (!player.dead) {
			// check if player out of bounds
			if (player.pos.x < player.min.x || player.pos.x > player.max.x || player.pos.y < player.min.y || player.pos.y > player.max.y) {
				KillPlayer();
			}

			// check if player collides with drone
			for (int i=0; i<4; i++) {
				if (player.CollidesWith(&drone[i])) {
					KillPlayer();
					break;
				}
			}
		}

		for (std::vector<EXPLOSION *>::iterator e = exp.begin(); e != exp.end(); ) {
			if (update_explosion(*e) == 1) {
				destroy_explosion(*e);
				e = exp.erase(e);
			}
			else {
				++e;
			}
		}
	}
}


void GameWidget::StartPlaying() {
	playing = true;
	NewGame();
}


void GameWidget::StopPlaying() {
	playing = false;
	holding_player = false;
	game_in_progress = false;
}


void GameWidget::Pause() {
	paused = !paused;
}


bool GameWidget::MsgChar(int c) {
	Widget::MsgChar(c);

	switch (c>>8) {
		case KEY_ESC:
		case KEY_P:
			Pause();
		break;

		case KEY_Q:
			if (paused) {
				Pause();
				StopPlaying();
				GetParent()->Close();
			}
		break;

		case KEY_C:
			if (paused) {
				Pause();
			}
		break;

		case KEY_SPACE:
		case KEY_ENTER:
			if (player.dead) {
				NewGame();
			}
		break;

		default:
			return false;
	};

	return true;
}


bool GameWidget::MsgWantfocus() {
	Widget::MsgWantfocus();
	return playing;
}



void GameWidget::MsgStart() {
	Widget::MsgStart();
	/*
	SetAnimationProperties(500, Animator::SLIDE_LEFT);
	ResetAnimation();
	Animate();
	*/
}


void GameWidget::MsgLPress() {
	Widget::MsgLPress();

	if (!paused && playing) {
		Point p = GetMousePos();
		if (player.pos.Distance2(p.x(), p.y()) < player.radius*player.radius) {
			holding_player = true;
		}
	}
}


void GameWidget::MsgLRelease() {
	Widget::MsgLRelease();
	holding_player = false;
}


void GameWidget::MsgMousemove(const Point &d) {
	Widget::MsgMousemove(d);

	if (holding_player && !player.dead) {
		player.pos.x += d.x();
		player.pos.y += d.y();
		game_in_progress = true;
	}
}


void GameWidget::NewGame() {
	tank[0].pos.x = 30;
	tank[0].pos.y = 240;
	tank[0].min.x = 30;
	tank[0].max.x = 30;
	tank[0].min.y = 48;
	tank[0].max.y = 432;
	tank[0].dir = itofix(192);

	tank[1].pos.x = 448;
	tank[1].pos.y = 240;
	tank[1].min.x = 448;
	tank[1].max.x = 448;
	tank[1].min.y = 48;
	tank[1].max.y = 432;
	tank[1].dir = itofix(64);

	tank[2].pos.x = 240;
	tank[2].pos.y = 32;
	tank[2].min.x = 48;
	tank[2].max.x = 432;
	tank[2].min.y = 32;
	tank[2].max.y = 32;
	tank[2].dir = itofix(0);

	tank[3].pos.x = 240;
	tank[3].pos.y = 448;
	tank[3].min.x = 48;
	tank[3].max.x = 432;
	tank[3].min.y = 448;
	tank[3].max.y = 448;
	tank[3].dir = itofix(128);

	for (int i=0; i<4; i++) {
		tank[i].rockets = &rockets;
	}

	static int posx[] = { 112, 368, 112, 368 };
	static int posy[] = { 112, 112, 368, 368 };

	for (int i=0; i<4; i++) {
		drone[i].min.x = 80;
		drone[i].max.x = 400;
		drone[i].min.y = 80;
		drone[i].max.y = 400;
		drone[i].pos.x = posx[i];
		drone[i].pos.y = posy[i];
		drone[i].sprite_index = i*2;
		drone[i].dir = itofix(0);
	}

	player.min.x = 80;
	player.max.x = 400;
	player.min.y = 80;
	player.max.y = 400;
	player.pos.x = 240;
	player.pos.y = 240;

	player.Init();
	for (int i=0; i<4; i++) {
		tank[i].Init();
		drone[i].Init();
	}

	rockets.clear();
	exp.clear();
	holding_player = false;
	game_in_progress = false;
	have_highscore = false;

	if (timePanel) {
		timePanel->time = 0;
	}
}
