#include <math.h>
#include "global.h"
#include "mainmenu.h"
#include "gamedialog.h"
#include "gamewidget.h"
#include "music_player.h"

//------------------------------------------------------------------------------------


// Placeholder for dialogs that have not yet been implemented.
class DialogNYI : public GameDialog {
	private:
		Background desktop;
		TextButton bBack;

	public:
		DialogNYI() : GameDialog() {
			desktop.Shape(0, 0, SCREEN_W, SCREEN_H);
			Add(desktop);

			int dy = 12;
			int y = 48-dy;
			int h = 10;
			int x = 10;
			int w = 80;

			bBack.Shape(x, y+=dy, w, h, 1);
			bBack.SetText("Not yet implemented!");
			bBack.MakeExit();
			bBack.AlignCentre();
			Add(bBack);

			Add(globalWidgets->title);
			Add(globalWidgets->lblBottom);
			Add(anim);
		}

		int Run() {
			Widget *obj = Execute(&bBack);
			return S_MAIN_MENU;
		}
};


class DialogAbout : public GameDialog {
	private:
		Background desktop;
		TextButton line[3];
		TextButton bBack;

	public:
		DialogAbout() : GameDialog() {
			desktop.Shape(0, 0, SCREEN_W, SCREEN_H);
			Add(desktop);

			int dy = 12;
			int y = 30-dy;
			int h = 10;
			int x = 10;
			int w = 80;

			static char *text[] = {
				"Dodger v%d.%02d",
				"By Miran Amon (c) 2006",
				"This is a TINS 06 game!"
			};

			for (int i=0; i<3; i++) {
				line[i].ClearFlag(D_AUTOSIZE);
				line[i].Shape(x, y+=dy, w, h, 1);
				line[i].AlignCentre();
				line[i].SetText(text[i]);
				line[i].Disable();
				Add(line[i]);
			}
			char buf[64];
			usprintf(buf, text[0], DODGER_VERSION_MAJOR, DODGER_VERSION_MINOR);
			line[0].SetText(buf);

			y += dy;
			bBack.Shape(x, y+=dy, w, h, 1);
			bBack.SetText("Back");
			bBack.MakeExit();
			bBack.AlignCentre();
			Add(bBack);

			Add(globalWidgets->title);
			Add(globalWidgets->lblBottom);
			Add(anim);
		}

		int Run() {
			Widget *obj = Execute(&bBack);
			return S_MAIN_MENU;
		}
};


class DialogHSC : public GameDialog {
	private:
		Background desktop;
		ListBoxEx lstHsc;
		TextButton bBack;
		HscHeader hscHeader;

	protected:
		void MsgInitSkin() {
			GameDialog::MsgInitSkin();

			for (int i=0; i<4; i++) {
				lstHsc.SetFont(1, i);
				lstHsc.SetFontColor(Color(0,192,0), Color::black, i);
				lstHsc.GetHeader()->SetFont(1, i);
				lstHsc.GetHeader()->SetFontColor(Color(0,192,0), Color::black, i);
			}
		}

	public:
		DialogHSC() : GameDialog() {
			desktop.Shape(0, 0, SCREEN_W, SCREEN_H);
			Add(desktop);

			lstHsc.Shape(40, 112, 560, 280);
			lstHsc.SetHeader(&hscHeader);
			lstHsc.InsertColumn("#", 0, 2);
			lstHsc.InsertColumn("Name", 1, 2);
			lstHsc.InsertColumn("Date", 2, 2);
			lstHsc.InsertColumn("Score", 3, 2);
			lstHsc.SetColumnWidth(0, 48);
			lstHsc.SetColumnWidth(1, 240);
			lstHsc.SetColumnWidth(2, 140);
			lstHsc.SetColumnWidth(3, 120);
			lstHsc.EnableColumnResizing(false);
			lstHsc.Disable();
			Add(lstHsc);

			bBack.Shape(10, 86, 80, 10, 1);
			bBack.SetText("Back");
			bBack.MakeExit();
			bBack.AlignCentre();
			Add(bBack);

			Add(globalWidgets->title);
			Add(globalWidgets->lblBottom);
			Add(anim);
		}

		~DialogHSC() {
			lstHsc.SetHeader(0);
		}

		int Run() {
			lstHsc.DeleteAllItems();
			for (int i=0; i<hsc->Count(); i++) {
				lstHsc.InsertItem(new HscItem(i%2));

				HSC::Entry e = hsc->GetEntry(i);
				static char buf[16];

				usprintf(buf, "%d", i+1);
				lstHsc.SetItem(i, 0, buf);
				lstHsc.SetItem(i, 1, e.name.c_str());
				lstHsc.SetItem(i, 2, e.date.c_str());
				int min = e.score/(60*100);
				int sec = e.score/100 - min*60;
				int msec = e.score - sec*100 - min*60*100;
				usprintf(buf, "%01d:%02d.%02d", min, sec, msec);
				lstHsc.SetItem(i, 3, buf);
			}
			Widget *obj = Execute(&bBack);
			return S_MAIN_MENU;
		}
};


class IntroScreen : public GameDialog {
	private:
		Background desktop;
		int counter;

		Label lbl[32];

	public:
		IntroScreen() : GameDialog() {
			Add(desktop);

			int dy = 8;
			int x = 10;
			int w = 620;
			int y = 120;
			int h = 360;

			for (int i=0; i<32; i++) {
				lbl[i].ClearFlag(D_AUTOSIZE);
				lbl[i].Shape(x, y+=dy, w, 14);
				Add(lbl[i]);
			}

			Add(anim);
		}

		int Run() {
			counter = 0;
			play_music(0);
			MAS::Settings::showMouse = false;
			Widget *obj = Execute();
			MAS::Settings::showMouse = true;
			return S_MAIN_MENU;
		}

		void MsgInitSkin() {
			GameDialog::MsgInitSkin();

			for (int i=0; i<32; i++) {
				lbl[i].SetFont(2, 0);
				lbl[i].SetFontColor(Color(0,192,0), Color::black, 0);
			}
		}

		void MsgTick() {
			GameDialog::MsgTick();

			if (::key[KEY_ENTER] || ::key[KEY_SPACE] || mouse_b) {
				Close();
			}

			static char *line1[] = {
				"                                       ______                                 ",
				" /'\\_/`\\  __                          /\\  _  \\                                ",
				"/\\      \\/\\_\\  _ __    __      ___    \\ \\ \\L\\ \\    ___ ___     ___     ___    ",
				"\\ \\ \\__\\ \\/\\ \\/\\`'__\\/'__`\\  /' _ `\\   \\ \\  __ \\ /' __` __`\\  / __`\\ /' _ `\\  ",
				" \\ \\ \\_/\\ \\ \\ \\ \\ \\//\\ \\L\\.\\_/\\ \\/\\ \\   \\ \\ \\/\\ \\/\\ \\/\\ \\/\\ \\/\\ \\L\\ \\/\\ \\/\\ \\ ",
				"  \\ \\_\\\\ \\_\\ \\_\\ \\_\\\\ \\__/.\\_\\ \\_\\ \\_\\   \\ \\_\\ \\_\\ \\_\\ \\_\\ \\_\\ \\____/\\ \\_\\ \\_\\",
				"   \\/_/ \\/_/\\/_/\\/_/ \\/__/\\/_/\\/_/\\/_/    \\/_/\\/_/\\/_/\\/_/\\/_/\\/___/  \\/_/\\/_/",
				"",
				"",
				"",
				"",
				"",
				"                                                  __             ",
				"                                                 /\\ \\__          ",
				"         _____   _ __    __    ____     __    ___\\ \\ ,_\\   ____  ",
				"        /\\ '__`\\/\\`'__\\/'__`\\ /',__\\  /'__`\\/' _ `\\ \\ \\/  /',__\\ ",
				"        \\ \\ \\L\\ \\ \\ \\//\\  __//\\__, `\\/\\  __//\\ \\/\\ \\ \\ \\_/\\__, `\\",
				"         \\ \\ ,__/\\ \\_\\\\ \\____\\/\\____/\\ \\____\\ \\_\\ \\_\\ \\__\\/\\____/",
				"          \\ \\ \\/  \\/_/ \\/____/\\/___/  \\/____/\\/_/\\/_/\\/__/\\/___/ ",
				"           \\ \\_\\                                                 ",
				"            \\/_/                                                 ",
				0
			};

			static char *line2[] = {
				"                 ______  ______   __  __  ____           __     ____    ",
				"                /\\__  _\\/\\__  _\\ /\\ \\/\\ \\/\\  _`\\       /'__`\\  /'___\\   ",
				"         __     \\/_/\\ \\/\\/_/\\ \\/ \\ \\ `\\\\ \\ \\,\\L\\_\\    /\\ \\/\\ \\/\\ \\__/   ",
				"       /'__`\\      \\ \\ \\   \\ \\ \\  \\ \\ , ` \\/_\\__ \\    \\ \\ \\ \\ \\ \\  _``\\ ",
				"      /\\ \\L\\.\\_     \\ \\ \\   \\_\\ \\__\\ \\ \\`\\ \\/\\ \\L\\ \\   \\ \\ \\_\\ \\ \\ \\L\\ \\",
				"      \\ \\__/.\\_\\     \\ \\_\\  /\\_____\\\\ \\_\\ \\_\\ `\\____\\   \\ \\____/\\ \\____/",
				"       \\/__/\\/_/      \\/_/  \\/_____/ \\/_/\\/_/\\/_____/    \\/___/  \\/___/ ",
				"",
				"",
				"",
				"",
				"",
				"                            __                  __                           ",
				"                           /\\ \\                /\\ \\__  __                    ",
				"     _____   _ __   ___    \\_\\ \\  __  __    ___\\ \\ ,_\\/\\_\\    ___     ___    ",
				"    /\\ '__`\\/\\`'__\\/ __`\\  /'_` \\/\\ \\/\\ \\  /'___\\ \\ \\/\\/\\ \\  / __`\\ /' _ `\\  ",
				"    \\ \\ \\L\\ \\ \\ \\//\\ \\L\\ \\/\\ \\L\\ \\ \\ \\_\\ \\/\\ \\__/\\ \\ \\_\\ \\ \\/\\ \\L\\ \\/\\ \\/\\ \\ ",
				"     \\ \\ ,__/\\ \\_\\\\ \\____/\\ \\___,_\\ \\____/\\ \\____\\\\ \\__\\\\ \\_\\ \\____/\\ \\_\\ \\_\\",
				"      \\ \\ \\/  \\/_/ \\/___/  \\/__,_ /\\/___/  \\/____/ \\/__/ \\/_/\\/___/  \\/_/\\/_/",
				"       \\ \\_\\                                                                 ",
				"        \\/_/                                                                 ",
				0
			};

			static char *line3[] = {
				"",
				"",
				"",
				"",
				"",
				"",
				"",
				"             ____                __                          ",
				"            /\\  _`\\             /\\ \\                         ",
				"            \\ \\ \\/\\ \\    ___    \\_\\ \\     __      __   _ __  ",
				"             \\ \\ \\ \\ \\  / __`\\  /'_` \\  /'_ `\\  /'__`\\/\\`'__\\",
				"              \\ \\ \\_\\ \\/\\ \\L\\ \\/\\ \\L\\ \\/\\ \\L\\ \\/\\  __/\\ \\ \\/ ",
				"               \\ \\____/\\ \\____/\\ \\___,_\\ \\____ \\ \\____\\\\ \\_\\ ",
				"                \\/___/  \\/___/  \\/__,_ /\\/___L\\ \\/____/ \\/_/ ",
				"                                          /\\____/            ",
				"                                          \\_/__/             ",
				0
			};

			static char *empty[] = {
				0
			};

			char **text = 0;

			++counter;
			switch (counter) {
				case 1:		text = line1;	break;
				case 200:	text = empty;	break;
				case 250:	text = line2;	break;
				case 450:	text = empty;	break;
				case 500:	text = line3;	break;
			};

			if (counter >= 500) {
				if ((counter/25)%2) {
					text = empty;
				}
				else {
					text = line3;
				}
			}

			if (text) {
				int i=0;
				while (text[i]) {
					lbl[i].SetText(text[i]);
					++i;
				}
				for (; i<32; i++) {
					lbl[i].SetText("");
				}
			}

			if (counter >= 1000) {
				Close();
			}
		}

		void MsgEnd() {
			GameDialog::MsgEnd();
			play_music(1);
		}
};


class OutroScreen : public GameDialog {
	private:
		Background desktop;
		TextButton lbl[2];
		int counter;

	public:
		OutroScreen() : GameDialog() {
			Add(desktop);

			int dy = 16;
			int y = 40-dy;
			int h = 10;
			int x = 0;
			int w = 100;

			char *lblText[] = {
				"Thank you for playing.",
				"See you in the next SpeedHack.",
			};

			for (int i=0; i<2; i++) {
				y += dy;

				lbl[i].Shape(x, y, w, h, 1);
				lbl[i].SetText(lblText[i]);
				lbl[i].Disable();
				lbl[i].AlignCentre();
				Add(lbl[i]);
			}

			Add(anim);
		}

		int Run() {
			counter = 0;
			Widget *obj = Execute();
			return S_EXIT_GAME;
		}

		void MsgTick() {
			GameDialog::MsgTick();

			if (::key[KEY_ENTER] || ::key[KEY_SPACE] || mouse_b) {
				Close();
			}

			++counter;
			if (counter >= 300) {
				Close();
			}
		}
};


class DialogSettings : public GameDialog {
	private:
		Background desktop;
		TextButton lbl[6];
		ChoiceButton btn[6];
		TextButton bBack;

	public:
		DialogSettings() : GameDialog() {
			desktop.Shape(0, 0, SCREEN_W, SCREEN_H);
			Add(desktop);

			int dy = 10;
			int y = 24-dy;
			int h = 10;
			int x = 0;
			int w = 100;

			char *lblText[] = {
				"fullscreen",
				"show framerate",
				"reduce CPU usage",
				"vsync",
				"sound",
				"music"
			};

			for (int i=0; i<6; i++) {
				y += dy;

				lbl[i].Shape(x, y, 2*w/3, h, 1);
				lbl[i].SetText(lblText[i]);
				lbl[i].Disable();
				lbl[i].AlignRight();
				Add(lbl[i]);

				btn[i].Shape(2*w/3+4, y, w/3 - 4, h, 1);
				btn[i].AlignLeft();
				Add(btn[i]);
			}

			for (int i=0; i<4; i++) {
				btn[i].choice.push_back("No");
				btn[i].choice.push_back("Yes");
			}

			for (int i=4; i<6; i++) {
				btn[i].choice.push_back("Off");
				btn[i].choice.push_back("10%%");
				btn[i].choice.push_back("20%%");
				btn[i].choice.push_back("30%%");
				btn[i].choice.push_back("40%%");
				btn[i].choice.push_back("50%%");
				btn[i].choice.push_back("60%%");
				btn[i].choice.push_back("70%%");
				btn[i].choice.push_back("80%%");
				btn[i].choice.push_back("90%%");
				btn[i].choice.push_back("100%%");
			}

			btn[0].selection = Settings::fullscreen ? 1 : 0;
			btn[1].selection = Settings::showFrameRate ? 1 : 0;
			btn[2].selection = Settings::yield ? 1 : 0;
			btn[3].selection = Settings::vsync ? 1 : 0;
			btn[4].selection = soundVolume;
			btn[5].selection = musicVolume;

			bBack.Shape(x, y+=dy, w, h, 1);
			bBack.SetText("Back");
			bBack.MakeExit();
			bBack.AlignCentre();
			Add(bBack);

			Add(globalWidgets->title);
			Add(globalWidgets->lblBottom);
			Add(anim);
		}

		int Run() {
			Widget *obj = Execute(&btn[0]);
			return S_MAIN_MENU;
		}

		void HandleEvent(Widget &obj, int msg, int arg1=0, int arg2=0) {
			GameDialog::HandleEvent(obj, msg, arg1, arg2);

			if (msg == MSG_ACTIVATE) {
				ChoiceButton *tmp = 0;
				int index = -1;

				for (int i=0; i<6; i++) {
					if (obj == btn[i]) {
						tmp = (ChoiceButton *)&obj;
						index = i;
						break;
					}
				}

				if (tmp) {
					if (arg1 == arg2 && arg1 == 0) {
						tmp->Increment();
					}

					switch (index) {
						case 0:		Settings::fullscreen = tmp->selection;
									ChangeResolution(Settings::gfxMode, Settings::fullscreen, Settings::screenWidth, Settings::screenHeight, Settings::colorDepth, Root());
									UnloadData();
									LoadData();
						break;
						case 1:		Settings::showFrameRate = tmp->selection;	break;
						case 2:		Settings::yield = tmp->selection;
									Settings::unlimitedFrameRate = tmp->selection ? 0 : 1;
						break;
						case 3:		Settings::vsync = tmp->selection;			break;
						case 4:		soundVolume = tmp->selection;
									Settings::soundVolume = soundVolume*25;
						break;
						case 5:		musicVolume = tmp->selection;				break;
					};
				}
			}
		}
};



//------------------------------------------------------------------------------------

// Now comes the core of our simple game menu system. The system is a finite state
// machine. It has a number of states where each has a dialog attached (except for
// the exit state). At all times the control is at exactly one of the states. For
// every iteration through the FSM the current state is evaluated and control is
// passed to the next state as indicated by the current state's return value.

void GameFramework() {
	// make all our states (dialogs)
	GameDialog *dlg[S_COUNT];
	dlg[S_INTRO] = new IntroScreen;
	dlg[S_MAIN_MENU] = new MainMenu;
	dlg[S_OPTIONS] = new DialogSettings;
	dlg[S_HIGHSCORES] = new DialogHSC;
	dlg[S_ABOUT] = new DialogAbout;
	dlg[S_PLAYING] = new MainGameDialog;
	dlg[S_OUTRO] = new OutroScreen;

	// give control to the entry point (the main menu)
	GameDialog *currentDialog = dlg[S_INTRO];

	// do the state machine in an endless loop
	while (true) {
		// evaluate the current state and see what it wants
DEBUG
		int nextState = currentDialog->Run();
DEBUG

		// if the next state is the exit state, break from the loop
		if (nextState == S_EXIT_GAME) {
DEBUG
			break;
		}
		// otherwise pass the control to the next state
		else {
DEBUG
			currentDialog = dlg[nextState];
DEBUG
		}
	}

	// clean up when we're done
DEBUG
	for (int i=0; i<S_COUNT; i++) {
DEBUG
		delete dlg[i];
	}
DEBUG
}


// the standard MASkinG main function
int main() {
	Error err = InstallMASkinG("allegro.cfg");
	if (err) {
		err.Report();
	}

	srand(time(0));

	atexit(&dumb_exit);
	dumb_register_stdfiles();

DEBUG
	set_window_title("Dodger");

	screenshot = create_screenshot("SHOT", "TGA");
	fps.Init(MAS::Settings::logicFrameRate);
	globalWidgets = new GlobalWidgets;
	LoadData();

DEBUG
	char full[1024];
	push_config_state();
	MakeFullPath(full, "allegro.cfg");
	set_config_file(full);
	soundVolume = get_config_int("Dodger", "soundVolume", soundVolume);
	musicVolume = get_config_int("Dodger", "musicVolume", musicVolume);
	Settings::soundVolume = soundVolume*25;
	pop_config_state();

	/* sin/cos look up tables */
	int c;
	for (c=0; c<LUT_SIZE; c++) {
		SIN[c] = sin(c*M_PI/128.0);
		COS[c] = cos(c*M_PI/128.0);
	}
	SIN[c] = sin(c*M_PI/128.0);
	COS[c] = cos(c*M_PI/128.0);

	MakeFullPath(full, "dodger.hsc");
	hsc = new HSC;
	hsc->Load(full);

DEBUG
	GameFramework();
DEBUG
	UnloadData();

DEBUG
	delete globalWidgets;

	MakeFullPath(full, "dodger.hsc");
	hsc->Save(full);
	delete hsc;

DEBUG
	flush_config_file();

DEBUG
	ExitMASkinG();
DEBUG

	push_config_state();
	MakeFullPath(full, "allegro.cfg");
	set_config_file(full);
	set_config_int("Dodger", "soundVolume", soundVolume);
	set_config_int("Dodger", "musicVolume", musicVolume);
	pop_config_state();

DEBUG
	return 0;
}
END_OF_MAIN();
